@@ -17,7 +17,7 @@
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
 #
-# $Id: rcpt-policy.tcl,v 1.5 2003/06/15 15:46:40 ian Exp $
+# $Id: rcpt-policy.tcl,v 1.7 2003/09/07 23:34:29 ian Exp $
 
 # userv service.  Input is new user policy file, lines like:
 #  <sending-site-pat>  <sending-addr-pat>  <receiving-addr-pat>  <result>
@@ -201,7 +201,7 @@ while 1 {
 	out "    if {[join $conds $condjoin]} \{"
     }
     if {![regexp \
-	    {^(45[012]|55[023]) \S.*$|^(unchecked|lax|nodelay|normal|bait)$} \
+ {^(?:errok\-)?(45[012]|55[023]) \S.*$|^(?:errok\-)?(unchecked|lax|nodelay|normal|bait)$} \
 	    $result]} {
 	syxerr "invalid result"
     }
