@@ -19,7 +19,7 @@
 # along with this program; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
 #
-# $Id: smtp.tcl,v 1.14.2.1 2004/01/14 22:56:33 ian Exp $
+# $Id: smtp.tcl,v 1.19 2004/01/14 22:59:13 ian Exp $
 
 
 ########## connection threads
@@ -65,8 +65,10 @@
 # smtpstyle         `smtp' or `esmtp'.
 # mtachan           channel to local MTA
 # helocmd           the command helo or ehlo that they used
+# heloexts          list of extensions we advertised in response to EHLO
 
 # Only set after MAIL FROM; unset/reset for new messages:
+#   ([re]set by ic_msg_resetvars)
 # rej_msg           list of reasons why we're rejecting this message
 # defer_msg         list of reasons why we're deferring this message
 #                    (only ever set before we get to DATA to avoid late defer)
@@ -81,6 +83,9 @@
 #                     and the same +lax (but avoid accepting normal after bait)
 #                    for this var, nodelay addrs are normal.  Repeats allowed.
 # mf_parms          parameters to MAIL FROM, if any
+# rcpts_errok_only  unset: no RCPT commands attempted since RSET
+#                   0: at least one RCPT attempted since RSET wasn't errok
+#                   1: all RCPTs (and there were some) were errok
 
 # add_bl            list of reasons to blacklist everyone to do with this msg
 #                    none may contain newline
@@ -253,7 +258,9 @@ proc intern_getsiteannoy {ra change} {
     global annoy_halflife annoy_grudge_max annoy_love_max
     global annoy_grumpy annoy_actout_max local_interface
 
-    if {[info exists local_interface($ra)]} { return {0 Submissive} }
+    if {[info exists local_interface($ra)]} {
+	return [list -$annoy_love_max Submissive]
+    }
 
     set ca [ds_get site-annoy $ra]
     set now [clock seconds]
@@ -310,6 +317,10 @@ thread_subproc ic commandresponse_maybefinal {evtype final response} {
 	immed - greeting { set annoychange 0 }
 	default { error "$evtype ?" }
     }
+    if {[info exists state(rcpts_errok_only)] && $state(rcpts_errok_only)} {
+	set min_delay $annoychange
+	set annoychange 0
+    }
     manyset [ic_getsiteannoy $annoychange] cv irritamt
     
     switch -exact $evtype {
@@ -334,18 +345,26 @@ thread_subproc ic commandresponse_maybefinal {evtype final response} {
 	delivery - immed { set delay 0 }
 	default { error "$evtype ?" }
     }
+    set irritpresent [irrit_present $irritamt]
+    set irritval "${cv}ms"
+    if {[info exists min_delay]} {
+	if {$delay < $min_delay} { set delay $min_delay }
+	append irritval -
+	append irritpresent -
+    }
+    set irritamtl [string tolower $irritamt]
     if {$delay > $annoy_actout_max} { set delay $annoy_actout_max }
     switch -exact $evtype {
 	major {
-	    logreject reject state command $response [string tolower $irritamt] ${cv}ms
+	    logreject reject state command $response $irritamtl $irritval
 	}
 	minor {
 	    if {$delay > 0} {
-		logreject notice state delay $response [string tolower $irritamt] ${cv}ms
+		logreject notice state delay $response $irritamtl $irritval
 	    }
 	}
 	rcpt-defer {
-	    logreject notice state rcpt-defer {} [string tolower $irritamt] ${cv}ms
+	    logreject notice state rcpt-defer {} $irritamtl $irritval
 	}
 	command - delivery - nopartresp - greeting - immed { }
 	default { error "$evtype ?" }
@@ -353,7 +372,7 @@ thread_subproc ic commandresponse_maybefinal {evtype final response} {
 
     if {[string length $response]} {
 	if {$delay > 0 || "$evtype" == "greeting"} {
-	    regsub {(?m)$} $response " \[[irrit_present $irritamt]\]" response
+	    regsub {(?m)$} $response " \[$irritpresent\]" response
 	}
     } else {
 	set delay 0
@@ -363,7 +382,7 @@ thread_subproc ic commandresponse_maybefinal {evtype final response} {
 }
 
 thread_subproc ic commandresponsedelay {delay nopartresp final response f_ix} {
-    global command_timeout annoy_partrespevery
+    global annoy_partrespevery
 
     if {$final<=0 && [ic_check_quitting]} { return }
 
@@ -388,11 +407,16 @@ thread_subproc ic commandresponsedelay {delay nopartresp final response f_ix} {
 	chanset_hide $state(chan) 1 1
 	threadio_puts ic $id $state(chan) $response tellquit_done tellquit_done
     } else {
-	set state(itoid) [thread_after ic $id $command_timeout timedout]
+	ic_setcommandtimeout
 	threadio_putsgets ic $id $state(chan) $response command_ok command_err
     }
 }
 
+thread_subproc ic setcommandtimeout {} {
+    global command_timeout
+    set state(itoid) [thread_after ic $id $command_timeout timedout]
+}
+
 thread_chainproc ic commandresponsedelay_part {delay final response f_ix} {
     global canonical_hostname fill_msgs
 
@@ -596,7 +620,14 @@ thread_chainproc ic command_ok {cmd} {
 				"501 Syntax error in recipient ($emsg)"
 		    } else {
 			set rtcmd "rcpt to:<[lp_quote $lp]@$dm>"
-			set atype [addr_classify $lp $dm state]
+			manyset [addr_classify $lp $dm state] atype errok
+
+			if {[info exists state(rcpts_errok_only)]} {
+			    set errok \
+				    [expr {$state(rcpts_errok_only) && $errok}]
+			}
+			set state(rcpts_errok_only) $errok
+
 			regexp {^[0-9a-z]+} $atype atype_summ
 			set kind $atype
 			switch -glob $atype_summ {
@@ -880,7 +911,6 @@ thread_subproc ic set_maildomain {ch hs} {
     set state(cmdomain) $ch
     set state(helostring) $hs
     ic_mtachan_open
-    threadio_commandresponse ic $id $state(mtachan) {} {} mta_greeting_ok {}
 }
 
 thread_subproc ic mtachan_open {} {
@@ -891,6 +921,7 @@ thread_subproc ic mtachan_open {} {
     set state(mtachan) [eval $lcmd]
     fconfigure $state(mtachan) -blocking false -translation {binary crlf}
     chanset_desc $state(mtachan) "$state(desc) / MTA"
+    threadio_commandresponse ic $id $state(mtachan) {} {} mta_greeting_ok {}
 }
 
 thread_chainproc ic mta_greeting_ok {data} {
@@ -917,25 +948,61 @@ thread_chainproc ic mta_helo_ok {data} {
     }
     set str "$canonical_hostname hello $state(ident)@$state(rh)"
     append str " (postmaster@$state(cmdomain)?)"
+    set exts {}
+    set silently [info exists state(heloexts)]
     if {"$state(helocmd)" == "helo"} {
-	set op "250 $str"
     } elseif {"$state(helocmd)" == "ehlo"} {
-	set op "250-$str\n"
 	foreach l [lrange [split $data "\n"] 1 end] {
-	    if {[regexp -nocase {^250[- ]([-a-z0-9]+)(.*)$} $l all keyword params]} {
+	    if {[regexp -nocase {^250[- ]([-a-z0-9]+)(.*)$} $l \
+		    all keyword params]} {
 		set params [string trim $params]
+		if {[string length $params]} { set params " $params" }
 		switch -exact -- [string tolower $keyword] {
 		    8bitmime - size {
-			append op "250-[string toupper $keyword] $params\n"
+			lappend exts "[string toupper $keyword]$params"
 		    }
 		}
 	    }
 	}
-	append op "250 PIPELINING"
+	manyset [ic_getsiteannoy 0] cv irritamttxt
+	if {$cv <= 0 || \
+		[info exists state(heloexts)] && \
+		[lsearch PIPELINING $state(heloexts)]>=0} {
+	    lappend exts PIPELINING
+	}
+	if {[info exists state(heloexts)]} {
+	    foreach item $state(heloexts) {
+		if {[lsearch -exact $exts $item]==-1} {
+		    # Oh, we advertised something before but not
+		    # the MTA isn't advertising it any more !  Doom.
+		    ic_commandfinalresponse minor \
+ "421 $canonical_hostname Internal error (EHLO extension lost: $item)"
+		    return
+		}
+	    }
+	} else {
+	    set state(heloexts) $exts
+	}
     } else {
 	error "internal error - ugh? $helocmd"
     }
-    ic_commandresponse nopartresp $op
+    if {!$silently} {
+	# mtachan_open was called as a result of HELO or EHLO command
+	set op {}
+	for {set i -1} {$i < [llength $exts]} {incr i} {
+	    set notlast [expr {$i+1 < [llength $exts]}]
+	    append op 250
+	    if {$notlast} { append op - } else { append op " " }
+	    if {$i>=0} { append op [lindex $exts $i] } else { append op $str }
+	    if {$notlast} { append op "\n" }
+	}
+	ic_commandresponse nopartresp $op
+    } else {
+	# mtachan_open was called as a result of another command dropping
+	# the MTA chan and needing to return to post-HELO state
+	ic_setcommandtimeout
+	threadio_gets ic $id $state(chan) command_ok command_err
+    }
 }
 
 thread_subproc ic mailfrom_fail {message} {
